#!/bin/bash

#####################################################################################
# Usage:
#
# This is intended to be run within the custom-helm container.
# It is expected that this script will be sourced by a calling script.
# It is also expected that the calling script will have used the `init.sh` script.
#
# Requirements:
#   * `/artifacts` directory which must contain the Python wheel
#   * `TF_VAR_processing_bucket_name` environment variable specifying the name of the
#      bucket to upload the resources to
#   * `GLUE_HANDLER_DEPLOY_MAP` environment variable that defines the mapping of glue
#      handler (locally) to S3 key
#   * `TF_VAR_default_nta_pop_lib_version` environment variable specifying the version of
#      the application library.
#####################################################################################

######
# This script deploys the necessary job resources to S3.
######

# Exit immediately on failure
set -e

######
# Functions
######

object_exists=0
does_s3_object_exist() {
    local num_results="$(aws s3api head-object \
    --bucket $TF_VAR_processing_bucket_name \
    --key $1 \
    2>&1 >/dev/null | grep -c 404)" || true
    if [ $num_results -eq 1 ]; then
        object_exists=0 # Object doesn't exist, detected a 404
    else
        object_exists=1 # Object exists, no 404 detected
    fi
}

######
# Main
######

print_header "Starting deployment of Glue job artifacts"

# Make sure the environment variable is set
if [ -z "$TF_VAR_processing_bucket_name" ] # If not set
then
    cprint "Missing environment variable `TF_VAR_processing_bucket_name`" "red"
    exit 1
fi

# Upload Python lib
cprint "Starting wheel upload process"
cprint "Using library version $TF_VAR_default_nta_pop_lib_version"
wheel_name=$(basename /artifacts/nta_pop_filter-$TF_VAR_default_nta_pop_lib_version-*.whl)
echo "wheel name: \"$wheel_name\""
does_s3_object_exist lib/$wheel_name
if [ $object_exists -eq 0 ]; then
    echo "Uploading wheel"
    aws s3 cp /artifacts/$wheel_name s3://$TF_VAR_processing_bucket_name/lib/
else
    echo "Skipping wheel upload -- object already exists in S3"
fi

# Upload bloom filter lib
cprint "Starting bloom filter upload process"
cprint "Using library version $TF_VAR_bloom_filter_lib_version"
jar_name=$(basename /artifacts/bloom-filter-lib-$TF_VAR_bloom_filter_lib_version.jar)
echo "jar name: \"$jar_name\""
does_s3_object_exist lib/$jar_name
if [ $object_exists -eq 0 ]; then
    echo "Uploading jar"
    aws s3 cp /artifacts/$jar_name s3://$TF_VAR_processing_bucket_name/lib/
else
    echo "Skipping jar upload -- object already exists in S3"
fi

# Upload glue handlers if config exists
cprint "Starting glue handler upload process"
echo "Checking for handler config map in \"$GLUE_HANDLER_DEPLOY_MAP\"."
if [ -s $GLUE_HANDLER_DEPLOY_MAP ]; then
    # Unpack the wheel
    unzip /artifacts/$wheel_name -d /artifacts/extracted/

    # Deploy each glue handler
    readarray -t glue_handlers < <(jq -c '.[]' $GLUE_HANDLER_DEPLOY_MAP)
    for item in "${glue_handlers[@]}"; do
        s3_key=$(jq -r '.s3_key' <<< "$item")
        versioned_s3_key=$(sed "s/.py/-$TF_VAR_default_nta_pop_lib_version.py/g" <<< "$s3_key")
        local_path="/artifacts/extracted/"$(jq -r '.local_path' <<< "$item")
        echo "Uploading $local_path to $versioned_s3_key"
        aws s3 cp $local_path s3://$TF_VAR_processing_bucket_name/$versioned_s3_key
    done
else
    echo "Skipping handler upload setup: No config found for file \"$GLUE_HANDLER_DEPLOY_MAP\"."
fi

print_header "Finished deployment of Glue job artifacts"
